DAT programName  byte "MergePad110820e", 0
CON
{{110820a Start working on program.
  
  Cogs used:
  0 MonitorPad
  1 Com0
  2 Com1
  3 LED 
  Four cogs used so far.


}}
CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  _EndOfMessagePad = "#"
  _EndOfMessageBlackBox = 13    ' Used by Black Box for received messages only.
                                ' Black Box will use _EndOfMessagePad for all messages
                                ' transmitted from the Black Box.
                                
  _BlackBoxMessagePrefix = "?"  ' So receiving unit know message isn't from any keypad.
  
  '' Pin asignments
 
  _PadARxPin = 0 
  _PadBRxPin = 1
  _PadCRxPin = 2
  _PadDRxPin = 3                                     
  _PadERxPin = 4 
  _PadFRxPin = 5
  _PadGRxPin = 6  
  _OutputRxPin = 31
  _OutputTxPin = 30  
  _LedPin = 23
    
  _PadBaud = 9600
  _LargeBufferSize = 256        ' This should be at least as large as the largest message
                                ' the Black Box is expected to receive.
                                ' I made it large since there is still plenty of memory.
                                
  _SerialPortGroupSize = 4      ' Serial port object is limited to four ports
  _SerialPortGroups = 2         ' Using to of the four port serial objects.

  ' Com enumeration
  #0, _PadA, _PadB, _PadC, _PadD, {second group} _PadE, _PadF, _PadG, _Output {0 - 7}
   
VAR

  long loopCounter
  long rxFlag[_SerialPortGroups * _SerialPortGroupSize]
  long oldRxFlag[_SerialPortGroups * _SerialPortGroupSize]
  
  byte tempBuffer[_LargeBufferSize]
  byte linesToExpect[_SerialPortGroups * _SerialPortGroupSize]
  ' So devices have multi line messages.
  ' lineToExpect makes it easier to handle multi-line messages.
  byte endOfMessageChar[_SerialPortGroups * _SerialPortGroupSize]
  
  
OBJ

  Com0 :  "pcFullDuplexSerial4FC512Flags110301a"        ' uses one cog
  Com1 :  "pcFullDuplexSerial4FC512Flags110301a2"       ' uses one cog
  Led : "Flash"                                         ' uses one cog (not really needed)
  
PUB Main | localIndex

  waitcnt(clkfreq * 3 + cnt)    ' Time to open PST

  repeat localIndex from _PadA to _Output
    linesToExpect[localIndex] := 1
    endOfMessageChar[localIndex] := _EndOfMessagePad
 
  endOfMessageChar[_Output] := _EndOfMessageBlackBox

  Led.Start(_LedPin, 150, 0)    ' hearbeat LED to show Propeller is working
    
  Com0.Init      
  Com0.AddPort(_PadA, _PadARxPin, -1, -1, -1, 0, 0, _PadBaud)
  Com0.AddPort(_PadB, _PadBRxPin, -1, -1, -1, 0, 0, _PadBaud)
  Com0.AddPort(_PadC, _PadCRxPin, -1, -1, -1, 0, 0, _PadBaud)
  Com0.AddPort(_PadD, _PadDRxPin, -1, -1, -1, 0, 0, _PadBaud)
  Com0.SetFlags(endOfMessageChar[_PadA], endOfMessageChar[_PadB], endOfMessageChar[_PadC], endOfMessageChar[_PadD], @rxFlag[0])
  Com0.Start
                                                             
  Com1.Init      
  Com1.AddPort(_PadE - _SerialPortGroupSize, _PadERxPin, -1, -1, -1, 0, 0, _PadBaud)
  Com1.AddPort(_PadF - _SerialPortGroupSize, _PadFRxPin, -1, -1, -1, 0, 0, _PadBaud)
  Com1.AddPort(_PadG - _SerialPortGroupSize, _PadGRxPin, -1, -1, -1, 0, 0, _PadBaud)
  Com1.AddPort(_Output - _SerialPortGroupSize, _OutPutRxPin, _OutPutTxPin, -1, -1, 0, 0, _PadBaud)
  Com1.SetFlags(endOfMessageChar[_PadE], endOfMessageChar[_PadF], endOfMessageChar[_PadG], endOfMessageChar[_Output], @rxFlag + (_SerialPortGroupSize * 4))
  Com1.Start                                                           
 
  waitcnt(clkfreq/10 + cnt)

  
  Tbug(string(_BlackBoxMessagePrefix, "programName = "))
  Tbug(@programName)
  TbugTx(_EndOfMessagePad)
  TbugTx(13)
  
  MonitorPads
  
PUB MonitorPads | localIndex, localSize, endRxFlag, localErrorFlag, localComIndex

  repeat     
    repeat localComIndex from _PadA to _Output
      if rxFlag[localComIndex] <> oldRxFlag[localComIndex]
        localSize := FlushBuffer(localComIndex)
        if localSize > 0
          Com1.tx(_Output - _SerialPortGroupSize, prefixList[localComIndex])
          localIndex := 0
          repeat localSize
            Com1.tx(_Output - _SerialPortGroupSize, tempBuffer[localIndex++])

PRI FlushBuffer(localBufferIndex) | localCharacter, localIndex, linesOut', previousWirelessCharacter

   if rxFlag[localBufferIndex] == oldRxFlag[localBufferIndex]                              
    Tbug(string(13, _BlackBoxMessagePrefix, " No buffer data, rxFlag["))
    TbugDec(localBufferIndex)
    Tbug(string("] = "))
    TbugDec(rxFlag[localBufferIndex])
    TbugTx(_EndOfMessagePad)
    return 0

  localIndex := 0
  linesOut := linesToExpect[localBufferIndex]
  repeat while linesOut > 0
    if localIndex => _LargeBufferSize
      Tbug(string(13, _BlackBoxMessagePrefix, "tempBuffer full!", _EndOfMessagePad))
      Tbug(string(13, _BlackBoxMessagePrefix, "This shouldn't happen.", _EndOfMessagePad)) 
      quit
    localCharacter := Rxcheck(localBufferIndex)
    if localCharacter == -1
      Tbug(string(13, _BlackBoxMessagePrefix, "rx buffer empty!!!", _EndOfMessagePad))
      Tbug(string(13, _BlackBoxMessagePrefix, "Data is missing!!!", _EndOfMessagePad))
      localCharacter := endOfMessageChar[localBufferIndex]
      Tbug(string(13, _BlackBoxMessagePrefix, "rxFlag["))
      TbugDec(localBufferIndex)
      Tbug(string("] = "))
      TbugDec(rxFlag[localBufferIndex])
      Tbug(string(", oldRxFlag["))
      TbugDec(localBufferIndex)
      Tbug(string("] = "))
      TbugDec(oldRxFlag[localBufferIndex])
      TbugTx(_EndOfMessagePad) 
    tempBuffer[localIndex] := localCharacter

    ' lineOut is needed for multi-line messages
    if localCharacter == endOfMessageChar[localBufferIndex]
      linesOut--
      
    localIndex++                ' localIndex keeps track of the length of the received message.

  tempBuffer[localIndex] := 0   ' This line is this not really needed.
                                ' Adding a zero terminator allows string opperations to be
                                ' preformed on tempBuffer.  (I don't think this
                                ' program uses any of these string opperations
                                ' on tempBuffer.)

  oldRxFlag[localBufferIndex]++ ' rxFlag and oldRxFlag should now be equal (if no new messages have been recieved).
  
  'SafeBug(@tempBuffer, localIndex) ' debug aid (if needed)
 
  result := localIndex          ' size of message received 

PRI Rxcheck(localComIndex)

  if localComIndex < _SerialPortGroupSize
    result := Com0.rxcheck(localComIndex)
  elseif localComIndex < _SerialPortGroupSize * 2
    result := Com1.rxcheck(localComIndex - _SerialPortGroupSize)
  else
    result := -1  
    
PRI Str(localComIndex, localStringPtr)

  if localComIndex < _SerialPortGroupSize
    Com0.str(localComIndex, localStringPtr)
  elseif localComIndex < _SerialPortGroupSize * 2
    Com1.str(localComIndex - _SerialPortGroupSize, localStringPtr) 
  else
    Tbug(string(13, _BlackBoxMessagePrefix, " Str error, Com Index out of range.", _EndOfMessagePad))
  
PRI Tx(localComIndex, localCharacter)

  if localComIndex < _SerialPortGroupSize
    Com0.tx(localComIndex, localCharacter)
  elseif localComIndex < _SerialPortGroupSize * 2
    Com1.tx(localComIndex - _SerialPortGroupSize, localCharacter) 
  else
    Tbug(string(13, _BlackBoxMessagePrefix, " Tx error, Com Index out of range.", _EndOfMessagePad))
  
PUB SafeBug(localPtr, localSize)

  repeat result from 0 to localSize - 1
    TbugTx(byte[localPtr + result])
      
PRI Tbug(localPtr)

  'Tv.str(localPtr)
  Com1.str(_Output - _SerialPortGroupSize, localPtr)
  

PRI TbugDec(localDec)

  'Tv.dec(localDec)
  Com1.dec(_Output - _SerialPortGroupSize, localDec)
  
PRI TbugTx(localCharacter)
'' Tbug... methods make it easier to add multiple debug
'' monitors (such as a TV).

  if (localCharacter > 31 and localCharacter < 127) or localCharacter == 13
    'Tv.out(localCharacter)
    Com1.tx(_Output - _SerialPortGroupSize, localCharacter)    
  else
    'Tv.out(60)
    'Tv.out(36)
    Com1.tx(_Output - _SerialPortGroupSize, 60)
    Com1.tx(_Output - _SerialPortGroupSize, 36)
    TbugHex(localCharacter, 2)
    'Tv.out(62)
    Com1.tx(_Output - _SerialPortGroupSize, 62)
   
PRI TbugHex(localHex, localSize)
 
  'Tv.hex(localHex, localSize)
  Com1.hex(_Output - _SerialPortGroupSize, localHex, localSize)

PRI TbugBin(localBin, localSize)

  'Tv.bin(localBin, localSize)
  Com1.bin(_Output - _SerialPortGroupSize, localBin, localSize)
  
DAT

prefixList    byte "ABCDEFG", _BlackBoxMessagePrefix
  